
c     The program "netCDF_bymonth" reads "high frequency" input fields,
c      and writes month-by-month means to an output file.
c     The output file must contain the proper specification, noteably
c      of the variable 'X' (the name of the variable that is specified
c      on the command line).

c     If no argument is given, the program terminates with an error
c     message which describes the correct syntax.

c     If the program terminates as intended, the file "ok_flag" is
c     touched in order to flag (seemingly) proper execution.


      program netCDF_bymonth

      implicit none

      include 'netcdf.inc'
      include 'nc_dims.inc'
      include 'missing_values.inc'

c     Parameter (#entries required for computation of StD)
      integer min_defined
      parameter(min_defined = 2)

c     Variables:
      integer*2 i2_field(nx,ny)
      integer   i4_field(nx,ny),n_data(nx,ny)
      real      r4_field(nx,ny)
      real*8    r8_field(nx,ny),sum(nx,ny)
      integer nc_status,org_length,origin_date(5),date(5),new_date(5),
     >        n_files,outno,file_no,monthno,yearno,
     >        n_records,start(4),count(4),i,x,y,z,t,no,
     >        nc_id,var_id,month_id,time_id,time_out_id,t_id,
     >        var_out_id,var_out_n_id,time_id_m,
     >        climate_dates_on,climate_dates_out,
     >        month_size,file_size,vrbl_size,size,
     >        n_dims,var_type,var_type_out,org_type,error
      real origin_sec,sec,timevalue,timefactor,
     >     offset,scale,offset_out,scale_out,limit,new_sec
      logical record_on,climate,first_field
      character*10 vrbl_name
      character*20 units,var_out,var_out_n
      character*120 org_files(100)
      character*120 month_file,present_file,unit_in,unit_out,d_t
c     For command line arguments:
      integer iargc,narg

c     Functions:
      integer str_len,str_pos
      real unit2unit


      narg = iargc()

      if (narg .ne. 1) then
         write(*,*)
         write(*,*)'Syntax error.'
         write(*,*)'  Correct syntax:'
         write(*,*)'    nc_bymonth /variable/ </filelist/'
         write(*,*)'  where "nc_bymonth" will add monthly fields to a"',
     >             ' a netCDF file.'
         write(*,*)'     /variable/  is the name of the variable under',
     >             ' consideration'
         write(*,*)'     /filelist/  is the name of a file that ',
     >             'contains'
         write(*,*)'                 * no. of file names on line 1'
         write(*,*)'                 * list of input file names on ',
     >             'following lines'
         write(*,*)'                 * output file name on the final ',
     >             'line'
         write(*,*)'Terminates.'
         write(*,*)
         stop
      end if


c     Set 'limit':
      limit = 0.999*abs(miss_val_r(1))

c     Initialize 'start', 'count', 'outno':
      start(1) = 1
      start(2) = 1
      count(1) = nx
      count(2) = ny
      count(3) = 1
      count(4) = 1
      outno    = 0

c     Read variable name:
      call getarg(1,vrbl_name)
      vrbl_size = str_len(vrbl_name)

c     Read no. of files containing data,and the file to put the climatology:
      read(5,*)n_files
      if (n_files .lt. 2 .or. n_files .gt. 101) go to 999
      write(*,*)
      write(*,*)'... read no. of file names:',n_files
      write(*,*)

c     Read the file names:
      do file_no = 1,n_files-1
         read(5,'(a120)')org_files(file_no)
      end do
      read(5,'(a120)')month_file
      month_size = str_len(month_file)

c     Open month-by-month file:
      nc_status = NF_OPEN(month_file(1:month_size), NF_WRITE, month_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,1)


c     --- START set output IDs (not lat-lon yet) ---

      var_out   = vrbl_name(1:vrbl_size)
      size      = str_len(var_out)
      nc_status = NF_INQ_VARID(month_id, var_out(1:size), var_out_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,2)

      nc_status = NF_INQ_VARID(month_id, 'time', time_out_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,4)

c     --- END set output IDs (not lat-lon yet) ---


c     --- START main loop ---

c     Consider one level at a time:
      do z = 1,max(1,nz)


c        (Re-)Initialize arrays for statistics:
         do y = 1,ny
            do x = 1,nx
               n_data(x,y) = 0
               sum(x,y) = dble(0.)
            end do
         end do

c        (Re-)Initialize 'first_field':
         first_field = .true.


c        --- START file loop ---

         do file_no = 1,n_files-1

c           Open netCDF file:
            present_file = org_files(file_no)
            file_size = str_len(present_file)
            nc_status = NF_OPEN(present_file(1:file_size), NF_NOWRITE,
     >                          nc_id)
            if (nc_status .ne. NF_NOERR) call handle_err(nc_status,11)

            write(*,*)'... opened file no.',file_no

c           Read, copy lon/lat?
            if (file_no .eq. 1 .and. z .eq. 1)
     >          call copy_lonlat(month_id,nc_id)

c           Check horizontal dimensions, set variable's ID:
            record_on = .true.
            if (nz .eq. 0) then
               call check_nc_xy(nc_id,sphere_on,nx,ny,record_on,1,
     >                          vrbl_name,var_id,error)
               if (error .ne. 0) stop
               n_dims = 3
            else if (z .eq. 1) then
               call check_nc_xyz(nc_id,sphere_on,nx,ny,nz,record_on,1,
     >                           vrbl_name,var_id,error)
               if (error .ne. 0) stop
               n_dims = 4
            end if

c           Read time units, no. of records etc. from original file:
            nc_status = NF_INQ_VARID(nc_id, 'time', time_id)
            if (nc_status .ne. NF_NOERR) call handle_err(nc_status,41)
            unit_in = ''
            nc_status = NF_GET_ATT_TEXT(nc_id, time_id, 'units',
     >                                  unit_in)
            if (nc_status .ne. NF_NOERR) call handle_err(nc_status,42)
c            nc_status = NF_GET_ATT_INT(nc_id, time_id,
c     >                                 'climate_dates_on',
c     >                                 climate_dates_on)
c            if (nc_status .ne. NF_NOERR) call handle_err(nc_status,43)
            climate_dates_on = 0
            nc_status = NF_INQ_DIMID(nc_id, 'time', t_id)
            if (nc_status .ne. NF_NOERR) call handle_err(nc_status,44)
            nc_status = NF_INQ_DIMLEN(nc_id, t_id, n_records)
            if (nc_status .ne. NF_NOERR) call handle_err(nc_status,45)

c           Set 'climate':
            if (climate_dates_on .eq. 1) then
               climate = .true.
            else
               climate = .false.
            end if

c           Find time origin, time units on input file:
            call string2date(unit_in,origin_date,origin_sec)
            size = str_pos(unit_in,1,' ') - 1
            units = unit_in(1:size)

            if (file_no .eq. 1 .and. z .eq. 1) then

c              Read time ID from output file:
               nc_status = NF_INQ_VARID(month_id, 'time', time_id_m)
               if (nc_status .ne. NF_NOERR)
     >            call handle_err(nc_status,51)

c              Check that use of climatological dates have been set:
               nc_status = NF_GET_ATT_INT(month_id, time_id_m,
     >                                    'climate_dates_on',
     >                                    climate_dates_out)
               if (nc_status .ne. NF_NOERR)
     >            call handle_err(nc_status,52)
               if (climate_dates_out .ne. 1) go to 998

c              Read units:
               unit_out = ''
               nc_status = NF_GET_ATT_TEXT(month_id, time_id_m, 'units',
     >                                     unit_out)
               if (nc_status .ne. NF_NOERR)
     >            call handle_err(nc_status,53)

c              Find time origin, time units on output file:
               call string2date(unit_out,new_date,new_sec)
               size = str_pos(unit_out,1,' ') - 1

c              Set time factor corresponding to one month:
               timefactor = unit2unit('months',1.,unit_out(1:size))

c              Read, check delta_t:
               d_t = ''
               nc_status = NF_GET_ATT_TEXT(month_id, time_id_m,
     >                                     'delta_t', d_t)
               if (nc_status .ne. NF_NOERR)
     >            call handle_err(nc_status,54)
               size = str_len(d_t)
               if (size .lt. 19) then
                  go to 997
               else if (d_t(1:19) .ne. '0000-01-00 00:00:00') then
c                 Override!
                  write(*,*)'Overriding original Dt, org: ',d_t(1:size)
                  d_t = '0000-01-00 00:00:00'
                  size = str_len(d_t)
                  write(*,*)'                        new: ',d_t(1:size)
                  nc_status = NF_PUT_ATT_TEXT(month_id, time_id_m,
     >                                        'delta_t',
     >                                        size, d_t(1:size))
                  if (nc_status .ne. NF_NOERR)
     >               call handle_err(nc_status,55)
               end if

            end if

c           Find the variable's type:
            call get_vartype(nc_id,month_id,var_id,var_out_id,file_no,z,
     >                       org_type,var_type,var_type_out,
     >                       offset_out,scale_out,offset,scale)


c           --- START time loop ---

            do no = 1, n_records

c              Read 'timevalue':
               nc_status = NF_GET_VAR1_REAL(nc_id, time_id, no,
     >                                      timevalue)
               if (nc_status .ne. NF_NOERR)
     >            call handle_err(nc_status,71)

c              Find the present date:
               call offset2date(climate,units,origin_date,
     >                          origin_sec,timevalue,date,sec)

               if (first_field) then

c                 Initialize 'yearno', 'monthno', 'n_data', 'sum':
                  yearno  = date(1)
                  monthno = date(2)
                  do y = 1,ny
                     do x = 1,nx
                        n_data(x,y) = 0
                        sum(x,y)    = dble(0.)
                     end do
                  end do

c                 Reset 'first_field':
                  first_field = .false.

               else if (date(2) .ne. monthno) then

c                 Compute, write, re-initialize:
                  call write_month(nx,ny,z,n_data,sum,r4_field,i2_field,
     >                             i4_field,new_date,date,var_type_out,
     >                             var_out_id,offset_out,scale_out,
     >                             yearno,monthno,timefactor,
     >                             month_id,time_id_m,outno)

               end if

c              (Re-)Set 'start':
               if (n_dims .eq. 3) then
                  start(3) = no
               else
                  start(3) = z
                  start(4) = no
               end if

c              Read next 2D (sub-)field, store & update:
               call read_set(nx,ny,i2_field,i4_field,r4_field,r8_field,
     >                       n_data,sum,nc_id,var_id,var_type,
     >                       start,count,offset,scale,limit)

            end do

c           --- END time loop ---


            nc_status = NF_CLOSE(nc_id)
            if (nc_status .ne. NF_NOERR) call handle_err(nc_status,301)

            write(*,*)'... closed file no.',file_no
            write(*,*)

         end do

c        --- END file loop ---


c        Compute, write, re-initialize:
         call write_month(nx,ny,z,n_data,sum,r4_field,i2_field,i4_field,
     >                    new_date,date,var_type_out,var_out_id,
     >                    offset_out,scale_out,yearno,monthno,
     >                    timefactor,month_id,time_id_m,outno)

      end do

c     --- END main loop ---


c     Close output file :
      nc_status = NF_CLOSE(month_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,991)

      write(*,*)
      write(*,*)'The program has apparently finished successfully.'
      write(*,*)

      open(41, file= 'ok_flag', status='unknown')
      close(41)

      go to 1000

 997  write(*,*)'Error in the program "nc_bymonth".'
      write(*,*)' Illegal date, too short!'
      write(*,*)'  Format should be             YYYY-MM-DD hh:mm:ss'
      write(*,*)'  (and possibly second decimals).'
      write(*,*)" Here, 'delta_t' was set to: ",d_t(1:size)
      write(*,*)'Terminates.'
      write(*,*)
      go to 1000

 998  write(*,*)
      write(*,*)'Error in the program "nc_bymonth".'
      write(*,*)' Switch for climatological set has not been set'
      write(*,*)' correctly (to 1), value:',climate_dates_out
      write(*,*)'Terminates.'
      write(*,*)
      go to 1000

 999  write(*,*)
      write(*,*)'Error in the program "nc_bymonth".'
      write(*,*)' Illegal no. of files specified, must be >1, <101.'
      write(*,*)' Specified value:',n_files
      write(*,*)'Terminates.'
      write(*,*)

 1000 end
